class Segment {

	public PVector offset;
	public PVector axis;
	public PVector control;


	public PVector storedOffset;
	public PVector storedAxis;
	public PVector storedControl;

	public int seed;

	public float angle;

	public int totalLines = 50;

	public PVector[] baseOrbit = new PVector[totalLines];
	public PVector[] startOrbit = new PVector[totalLines];
	public PVector[] endOrbit = new PVector[totalLines];

	public PVector startRelativeVec;
	public PVector endRelativeVec;
	public PVector startCrossRelativeVec;
	public PVector endCrossRelativeVec;
	public PVector pseudoRelativeVec;

	public PVector startAbsoluteVec;
	public PVector endAbsoluteVec;
	public PVector startCrossAbsoluteVec;
	public PVector endCrossAbsoluteVec;
	public PVector pseudoAbsoluteVec;
		
	color from = color(255, 0, 102);
	color to = color(0, 102, 153);

	public float scalar = 50;

	public Segment() {
		int radius = 25;

		axis 	= new PVector( 25, 0, 0 );
		offset 	= new PVector( 0,0,0);
		control = new PVector( 0, 10, 0 );
		
		storedAxis = new PVector( 0, random(25), random(25) );
		storedOffset = offset.get();
		storedControl = control.get();
		
		createRelatives();
		resetRelatives();
		createAbsolutes();
	}

	public void createRelatives(){
		int radius = 300;
		float half_radius = radius*.5;
		startRelativeVec = new PVector( 0, 0, 0 );
		endRelativeVec = new PVector( random(radius)-half_radius, random(radius)-half_radius, random(radius)-half_radius );
		pseudoRelativeVec = new PVector( 0, 0, scalar );
		startCrossAbsoluteVec = new PVector( 0, 0, 0 );
		endCrossAbsoluteVec = new PVector( 0, 0, 0 );
	}

	public void resetRelatives(){
		
		startCrossRelativeVec = endRelativeVec.cross(pseudoRelativeVec);
		startCrossRelativeVec.normalize();
		startCrossRelativeVec.mult(scalar);

		endCrossRelativeVec = startCrossRelativeVec.get();
		endCrossRelativeVec.add(endRelativeVec);

	}
	
	
	public void createAbsolutes(){
		startAbsoluteVec = startRelativeVec.get();
		endAbsoluteVec = endRelativeVec.get();
		startCrossAbsoluteVec = startCrossRelativeVec.get();
		endCrossAbsoluteVec = endCrossRelativeVec.get();
		pseudoAbsoluteVec = pseudoRelativeVec.get();
	}
	
	public void updateAbsolutes(PVector offset){
		startAbsoluteVec.add(offset);
		endAbsoluteVec.add(offset);
		startCrossAbsoluteVec.add(offset);
		endCrossAbsoluteVec.add(offset);
		pseudoAbsoluteVec.add(offset);
	}
	
	public void parse() {
		for(int i=0;i<totalLines;i++) {
			angle = 360*float(i)/float(totalLines) +360*(1-float(seed)/float(totalLines));
			// startOrbit[i] = rotateAroundArbitraryAxis( offset, control, axis, angle );
			
			// baseOrbit[i] = rotateAroundArbitraryAxis( startRelativeVec, startCrossRelativeVec, endRelativeVec, angle );

			startOrbit[i] = rotateAroundArbitraryAxis( startAbsoluteVec, startCrossAbsoluteVec, endAbsoluteVec, angle );

			// startOrbit[i] = baseOrbit[i].get();
			// startOrbit[i].add(startAbsoluteVec);
			
			endOrbit[i] = startOrbit[i].get();
			endOrbit[i].add(endRelativeVec);
		}         
	}

	public void draw() {
		// drawWidthArray(startOrbit,#FFFFFF);
		drawCircleOutlineArray(startOrbit,#FFFFFF);
		drawCircleOutlineArray(endOrbit,#FFFFFF);
		
		PVector curr = new PVector(0,0,0);
		PVector prev = new PVector(0,0,0);  
		int lines = endOrbit.length;
		int i;
		for(i=0;i<lines;i++) {
			curr = startOrbit[i].get();
			prev = endOrbit[i].get();
			drawLine(prev,curr,1,#FFFFFF,51);
		}
	}
	
	public void drawGuides() {
		
		drawPoint(endAbsoluteVec,3,#FFFFFF,255);
		drawLine(startAbsoluteVec,endAbsoluteVec,1,#FFFFFF,102);

		drawLine(startAbsoluteVec,pseudoAbsoluteVec,1,#FFFFFF,102);

		drawPoint(startCrossAbsoluteVec,3,#FF00FF,255);
		drawLine(startAbsoluteVec,startCrossAbsoluteVec,1,#FF00FF,102);

		drawPoint(endCrossAbsoluteVec,3,#00FF00,255);
		drawLine(endAbsoluteVec,endCrossAbsoluteVec,1,#00FF00,102);

	}
	void drawCircleOutlineArray(PVector[] orbiter, color colorParam) {
		PVector curr = new PVector(0,0,0);
		PVector prev = new PVector(0,0,0);  
		int lines = orbiter.length;
		int i;
		
		prev = orbiter[0].get();
		curr = orbiter[lines-1].get();
		drawLine(prev,curr,1,#FFFFFF,51);

		for(i=1;i<lines;i++) {
			prev = curr.get();
			curr = orbiter[i].get();
			drawLine(prev,curr,1,#FFFFFF,51);
		}
		
	}
	
	
	void drawWidthArray(PVector[] orbiter, color colorParam) {

		PVector curr = new PVector(0,0,0);
		PVector prev = new PVector(0,0,0);  
		int lines = orbiter.length;
		float percentage;
		float colorShifter = sin( ((float(seed) / float(totalSegments) + frameCount*.001) * TWO_PI))*.5+.5;
		strokeWeight(1);
		color interColor = lerpColor(from, to,  colorShifter);

		for(int i=1;i<lines;i++) {
			curr = orbiter[i].get();
			prev = orbiter[i-1].get();

			percentage = (float(i)/float(lines)) * 255;

			noStroke();

			fill(interColor, percentage);
			beginShape(TRIANGLE_FAN);
			vertex(prev.x,prev.y,prev.z);
			vertex(curr.x,curr.y,curr.z);
			vertex(startAbsoluteVec.x,startAbsoluteVec.y,startAbsoluteVec.z);
			vertex(prev.x,prev.y,prev.z);
			endShape();

		}
	}

	public void drawPoint(PVector pointParam, int strokeParam, color colorParam, float opacityParam) {
		strokeWeight(strokeParam);
		stroke(colorParam,opacityParam);
		point( pointParam.x,pointParam.y,pointParam.z);
	}

	public void drawLine(PVector pointAParam,PVector pointBParam, float strokeParam, color colorParam, float opacityParam) {
		strokeWeight(strokeParam);
		stroke(colorParam,opacityParam);
		line( pointAParam.x,pointAParam.y,pointAParam.z,pointBParam.x,pointBParam.y,pointBParam.z);
	}
}

